@@ -643,7 +643,8 @@ gdict_command_new (GdictCommandType cmd_type)
   
   g_assert (IS_VALID_CMD (cmd_type));
   
-  retval = g_new0 (GdictCommand, 1);
+  retval = g_slice_new0 (GdictCommand);
+  
   retval->cmd_type = cmd_type;
   retval->state = S_START;
   
@@ -685,7 +686,7 @@ gdict_command_free (GdictCommand *cmd)
   if (cmd->data_destroy)
     cmd->data_destroy (cmd->data);
   
-  g_free (cmd);
+  g_slice_free (GdictCommand, cmd);
 }
 
 /* push @command into the head of the commands queue; the command queue is
@@ -1287,6 +1288,7 @@ gdict_client_context_parse_line (GdictClientContext *context,
       priv->command->state = S_FINISH;
       break;
     default:
+      gdict_debug ("non error code: %d\n", priv->status_code);
       break;
     }
 
@@ -1333,6 +1335,11 @@ gdict_client_context_parse_line (GdictClientContext *context,
       return TRUE;
     }
 
+  gdict_debug ("check command %d ('%s')[state:%d]\n",
+	       priv->command->cmd_type,
+	       dict_command_strings[priv->command->cmd_type],
+	       priv->command->state);
+
   /* check command type */
   switch (priv->command->cmd_type)
     {
@@ -1352,6 +1359,7 @@ gdict_client_context_parse_line (GdictClientContext *context,
           
           gdict_debug ("server replied: %d databases found\n", atoi (p));
           
+	  gdict_debug ("emitting lookup-start\n");
           g_signal_emit_by_name (context, "lookup-start");
         }
       else if (0 == strcmp (buffer, "."))
@@ -1386,6 +1394,7 @@ gdict_client_context_parse_line (GdictClientContext *context,
           db = _gdict_database_new (name);
           db->full_name = g_strdup (full);
           
+	  gdict_debug ("emitting database-found\n");
           g_signal_emit_by_name (context, "database-found", db);
           
           gdict_database_unref (db);
@@ -1404,6 +1413,7 @@ gdict_client_context_parse_line (GdictClientContext *context,
           
           gdict_debug ("server replied: %d strategies found\n", atoi (p));
           
+	  gdict_debug ("emitting lookup-start\n");
           g_signal_emit_by_name (context, "lookup-start");
         }
       else if (0 == strcmp (buffer, "."))
@@ -1435,6 +1445,7 @@ gdict_client_context_parse_line (GdictClientContext *context,
           strat = _gdict_strategy_new (name);
           strat->description = g_strdup (desc);
           
+	  gdict_debug ("emitting strategy-found\n");
           g_signal_emit_by_name (context, "strategy-found", strat);
           
           gdict_strategy_unref (strat);
@@ -1459,6 +1470,7 @@ gdict_client_context_parse_line (GdictClientContext *context,
           priv->command->data = def;
           priv->command->data_destroy = (GDestroyNotify) gdict_definition_unref;
 
+	  gdict_debug ("emitting lookup-start\n");
           g_signal_emit_by_name (context, "lookup-start");
         }
       else if (priv->status_code == GDICT_STATUS_WORD_DB_NAME)
@@ -1505,7 +1517,7 @@ gdict_client_context_parse_line (GdictClientContext *context,
           
           def = (GdictDefinition *) priv->command->data;
 
-	  gdict_debug ("{ word = '%s', db_name = '%s', db_full = '%s' }",
+	  gdict_debug ("{ word .= '%s', db_name .= '%s', db_full .= '%s' }\n",
 		       word,
 		       db_name,
 		       db_full);
@@ -1533,6 +1545,7 @@ gdict_client_context_parse_line (GdictClientContext *context,
 	  /* store the numer of definitions */
 	  num = def->total;
           
+	  gdict_debug ("emitting definition-found\n");
           g_signal_emit_by_name (context, "definition-found", def);
           
           gdict_definition_unref (def);
@@ -1549,8 +1562,9 @@ gdict_client_context_parse_line (GdictClientContext *context,
           if (!priv->command->buffer)
             priv->command->buffer = g_string_new (NULL);
           
-          /* TODO - collapse '..' to '.' */
+	  gdict_debug ("appending line:\n %s\n", buffer);
           
+          /* TODO - collapse '..' to '.' */
           g_string_append_printf (priv->command->buffer, "%s\n", buffer);
         }
       break;
@@ -1567,6 +1581,7 @@ gdict_client_context_parse_line (GdictClientContext *context,
           
           gdict_debug ("server replied: %d matches found\n", atoi (p));
 
+	  gdict_debug ("emitting lookup-start\n");
           g_signal_emit_by_name (context, "lookup-start");
         }
       else if (0 == strcmp (buffer, "."))
@@ -1598,6 +1613,7 @@ gdict_client_context_parse_line (GdictClientContext *context,
           match = _gdict_match_new (word);
           match->database = g_strdup (db_name);
           
+	  gdict_debug ("emitting match-found\n");
           g_signal_emit_by_name (context, "match-found", match);
           
           gdict_match_unref (match);
@@ -1613,10 +1629,11 @@ gdict_client_context_parse_line (GdictClientContext *context,
 
 /* retrieve the status code from the server response line */
 static gint
-get_status_code (const gchar *line)
+get_status_code (const gchar *line,
+		 gint         old_status)
 {
   gchar *status;
-  gint retval;
+  gint possible_status, retval;
   
   if (strlen (line) < 3)
     return 0;
@@ -1627,12 +1644,37 @@ get_status_code (const gchar *line)
     return 0;
   
   status = g_strndup (line, 3);
-  retval = atoi (status);
+  possible_status = atoi (status);
   g_free (status);
+
+  /* status whitelisting: sometimes, a database *cough* moby-thes *cough*
+   * might return a number as first word; we do a small check here for
+   * invalid status codes based on the previously set status; we don't check
+   * the whole line, as we need only to be sure that the status code is
+   * consistent with what we expect.
+   */
+  switch (old_status)
+    {
+    case GDICT_STATUS_WORD_DB_NAME:
+    case GDICT_STATUS_N_MATCHES_FOUND:
+      if (possible_status == GDICT_STATUS_OK)
+	retval = possible_status;
+      else
+        retval = 0;
+      break;
+    case GDICT_STATUS_N_DEFINITIONS_RETRIEVED:
+      if (possible_status == GDICT_STATUS_WORD_DB_NAME)
+	retval = possible_status;
+      else
+	retval = 0;
+      break;
+    default:
+      retval = possible_status;
+      break;
+    }
   
   return retval;
 }  
-  
 
 static gboolean
 gdict_client_context_io_watch_cb (GIOChannel         *channel,
@@ -1719,13 +1761,15 @@ gdict_client_context_io_watch_cb (GIOChannel         *channel,
       /* truncate the line terminator before parsing */
       line[term] = '\0';
       
-      status_code = get_status_code (line);
+      status_code = get_status_code (line, priv->status_code);
       if ((status_code == 0) || (GDICT_IS_VALID_STATUS_CODE (status_code)))
         {
           priv->status_code = status_code;
           
           gdict_debug ("new status = '%d'\n", priv->status_code);
         }
+      else
+        priv->status_code = GDICT_STATUS_INVALID;
       
       /* notify changes only for valid status codes */
       if (priv->status_code != GDICT_STATUS_INVALID)
