@@ -2,6 +2,7 @@
 #include "screenshot-dialog.h"
 #include "screenshot-save.h"
 #include <gnome.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 
 
 static GtkTargetEntry drag_types[] =
@@ -169,6 +170,8 @@ screenshot_dialog_new (GdkPixbuf *screenshot,
   gint width, height;
   char *current_folder;
   char *current_name;
+  char *ext;
+  gint pos;
   GnomeVFSURI *tmp_uri;
   GnomeVFSURI *parent_uri;
 
@@ -192,7 +195,7 @@ screenshot_dialog_new (GdkPixbuf *screenshot,
 				       GTK_MESSAGE_ERROR,
 				       GTK_BUTTONS_OK,
 				       _("Glade file for the screenshot program is missing.\n"
-					 "Please check your installation of gnome-panel"));
+					 "Please check your installation of gnome-utils"));
       gtk_dialog_run (GTK_DIALOG (dialog));
       gtk_widget_destroy (dialog);
       exit (1);
@@ -210,14 +213,13 @@ screenshot_dialog_new (GdkPixbuf *screenshot,
   dialog->filename_entry = glade_xml_get_widget (dialog->xml, "filename_entry");
   file_chooser_box = glade_xml_get_widget (dialog->xml, "file_chooser_box");
 
-  dialog->save_widget = gtk_file_chooser_button_new (_("Select a directory"), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
+  dialog->save_widget = gtk_file_chooser_button_new (_("Select a folder"), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
   gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (dialog->save_widget), FALSE);
   gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (dialog->save_widget), current_folder);
   gtk_entry_set_text (GTK_ENTRY (dialog->filename_entry), current_name);
 
   gtk_box_pack_start (GTK_BOX (file_chooser_box), dialog->save_widget, TRUE, TRUE, 0);
   g_free (current_folder);
-  g_free (current_name);
 
   gtk_widget_set_size_request (preview_darea, width, height);
   gtk_aspect_frame_set (GTK_ASPECT_FRAME (aspect_frame), 0.0, 0.5,
@@ -243,6 +245,21 @@ screenshot_dialog_new (GdkPixbuf *screenshot,
 
   gtk_widget_show_all (toplevel);
 
+  /* select the name of the file but leave out the extension if there's any;
+   * the dialog must be realized for select_region to work
+   */
+  ext = g_utf8_strchr (current_name, -1, '.');
+  if (ext && ext != current_name)
+    pos = ext - current_name;
+  else
+    pos = -1;
+
+  gtk_editable_select_region (GTK_EDITABLE (dialog->filename_entry),
+			      0,
+			      pos);
+  
+  g_free (current_name);
+
   return dialog;
 }
 
@@ -271,13 +288,17 @@ screenshot_dialog_get_uri (ScreenshotDialog *dialog)
 {
   gchar *folder;
   const gchar *file_name;
-  gchar *uri;
+  gchar *uri, *file, *tmp;
 
   folder = gtk_file_chooser_get_current_folder_uri (GTK_FILE_CHOOSER (dialog->save_widget));
   file_name = gtk_entry_get_text (GTK_ENTRY (dialog->filename_entry));
 
-  uri = g_build_filename (folder, file_name, NULL);
+  tmp = g_filename_from_utf8 (file_name, -1, NULL, NULL, NULL);
+  file = gnome_vfs_escape_host_and_path_string (tmp);
+  uri = g_build_filename (folder, file, NULL);
   g_free (folder);
+  g_free (tmp);
+  g_free (file);
 
   return uri;
 }
